"""Test opening of file format variations"""
import pathlib
import numpy as np

from nanite import IndentationGroup

datadir = pathlib.Path(__file__).resolve().parent / "data"


def test_process_flipsign():
    # This is a curve extracted from a map file. When loading it
    # with nanite, the sign of the force curve was flipped.
    flipped = datadir / "flipsign_2015.05.22-15.31.49.352.jpk-force"
    apret = IndentationGroup(flipped)[0]
    apret.apply_preprocessing(["compute_tip_position",
                               "correct_force_offset",
                               "correct_tip_offset",
                               "correct_split_approach_retract"])

    apret.fit_model(model_key="hertz_para", weight_cp=False)
    assert np.allclose(apret.fit_properties["params_fitted"]["E"].value,
                       5230.0087777251456)


if __name__ == "__main__":
    # Run all tests
    loc = locals()
    for key in list(loc.keys()):
        if key.startswith("test_") and hasattr(loc[key], "__call__"):
            loc[key]()



from . import model_conical_indenter  # noqa: F401
from . import model_hertz_paraboloidal  # noqa: F401
from . import model_hertz_three_sided_pyramid  # noqa: F401
from . import model_sneddon_spherical  # noqa: F401
from . import model_sneddon_spherical_approximation  # noqa: F401


def get_model_by_name(name):
    """Convenience function to obtain a model by name instead of by key"""
    for key in models_available:
        if models_available[key].model_name == name:
            return models_available[key]
    else:
        raise KeyError("No model with name {}".format(name))


def get_parm_name(model_key, parm_key):
    """Get human readable parameter label

    Parameters
    ----------
    model_key: str
        The model key (e.g. "hertz_cone")
    parm_key: str
        The parameter key (e.g. "E")

    Returns
    -------
    parm_name: str
        The parameter name (e.g. "Young's Modulus")
    """
    model = models_available[model_key]
    idx = model.parameter_keys.index(parm_key)
    return model.parameter_names[idx]


def get_init_parms(model_key):
    """Get initial fit parameters for a model"""
    model = models_available[model_key]
    parms = model.get_parameter_defaults()
    return parms


models_available = {}

# Populate list of available fit models
loc = locals().copy()
for item in loc:
    if item.startswith("model_"):
        models_available[loc[item].model_key] = loc[item]
